@@ -1,5 +1,6 @@
 """Tests for TLS support."""
 
+import contextlib
 import functools
 import http.client
 import json
@@ -16,6 +17,13 @@ import pytest
 import OpenSSL.SSL
 import requests
 import trustme
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.serialization import (
+    BestAvailableEncryption,
+    Encoding,
+    PrivateFormat,
+    load_pem_private_key,
+)
 
 from .._compat import (
     IS_ABOVE_OPENSSL10,
@@ -24,12 +32,14 @@ from .._compat import (
     IS_LINUX,
     IS_MACOS,
     IS_PYPY,
+    IS_SOLARIS,
     IS_WINDOWS,
+    SYS_PLATFORM,
     bton,
     ntob,
     ntou,
 )
-from ..server import HTTPServer, get_ssl_adapter_class
+from ..server import Gateway, HTTPServer, get_ssl_adapter_class
 from ..testing import (
     ANY_INTERFACE_IPV4,
     ANY_INTERFACE_IPV6,
@@ -136,6 +146,12 @@ def make_tls_http_server(bind_addr, ssl_adapter, request):
     return httpserver
 
 
+@pytest.fixture(scope='session')
+def private_key_password():
+    """Provide hardcoded password for private key."""
+    return 'криївка'
+
+
 @pytest.fixture
 def tls_http_server(request):
     """Provision a server creator as a fixture."""
@@ -176,6 +192,34 @@ def tls_certificate_private_key_pem_path(tls_certificate):
         yield cert_key_pem
 
 
+@pytest.fixture
+def tls_certificate_passwd_private_key_pem_path(
+    tls_certificate,
+    private_key_password,
+    tmp_path,
+):
+    """Return a certificate private key PEM file path."""
+    key_as_bytes = tls_certificate.private_key_pem.bytes()
+    private_key_object = load_pem_private_key(
+        key_as_bytes,
+        password=None,
+        backend=default_backend(),
+    )
+
+    encrypted_key_as_bytes = private_key_object.private_bytes(
+        encoding=Encoding.PEM,
+        format=PrivateFormat.PKCS8,
+        encryption_algorithm=BestAvailableEncryption(
+            password=private_key_password.encode('utf-8'),
+        ),
+    )
+
+    key_file = tmp_path / 'encrypted-private-key.pem'
+    key_file.write_bytes(encrypted_key_as_bytes)
+
+    return key_file
+
+
 def _thread_except_hook(exceptions, args):
     """Append uncaught exception ``args`` in threads to ``exceptions``."""
     if issubclass(args.exc_type, SystemExit):
@@ -225,7 +269,7 @@ def thread_exceptions():
         'pyopenssl',
     ),
 )
-def test_ssl_adapters(
+def test_ssl_adapters(  # pylint: disable=too-many-positional-arguments
     http_request_timeout,
     tls_http_server,
     adapter_type,
@@ -294,6 +338,7 @@ def test_ssl_adapters(
     reason='Fails under PyPy in CI for unknown reason',
     strict=False,
 )
+# pylint: disable-next=too-many-positional-arguments
 def test_tls_client_auth(  # noqa: C901, WPS213  # FIXME
     # FIXME: remove twisted logic, separate tests
     http_request_timeout,
@@ -498,6 +543,7 @@ def test_tls_client_auth(  # noqa: C901, WPS213  # FIXME
         (ssl.CERT_REQUIRED, True),
     ),
 )
+# pylint: disable-next=too-many-positional-arguments
 def test_ssl_env(  # noqa: C901  # FIXME
     thread_exceptions,
     recwarn,
@@ -664,6 +710,7 @@ def test_https_over_http_error(http_server, ip_addr):
     ),
 )
 @pytest.mark.flaky(reruns=3, reruns_delay=2)
+# pylint: disable-next=too-many-positional-arguments
 def test_http_over_https_error(
     http_request_timeout,
     tls_http_server,
@@ -681,6 +728,30 @@ def test_http_over_https_error(
     if issue_225:
         pytest.xfail('Test fails in Travis-CI')
 
+    if IS_LINUX:
+        expected_error_code, expected_error_text = (
+            104,
+            'Connection reset by peer',
+        )
+    elif IS_MACOS:
+        expected_error_code, expected_error_text = (
+            54,
+            'Connection reset by peer',
+        )
+    elif IS_SOLARIS:
+        expected_error_code, expected_error_text = (
+            None,
+            'Remote end closed connection without response',
+        )
+    elif IS_WINDOWS:
+        expected_error_code, expected_error_text = (
+            10054,
+            'An existing connection was forcibly closed by the remote host',
+        )
+    else:
+        expected_error_code, expected_error_text = None, None
+        pytest.skip(f'{SYS_PLATFORM} is unsupported')  # pragma: no cover
+
     tls_adapter_cls = get_ssl_adapter_class(name=adapter_type)
     tls_adapter = tls_adapter_cls(
         tls_certificate_chain_pem_path,
@@ -721,25 +792,143 @@ def test_http_over_https_error(
             timeout=http_request_timeout,
         )
 
-    if IS_LINUX:
-        expected_error_code, expected_error_text = (
-            104,
-            'Connection reset by peer',
-        )
-    if IS_MACOS:
-        expected_error_code, expected_error_text = (
-            54,
-            'Connection reset by peer',
-        )
-    if IS_WINDOWS:
-        expected_error_code, expected_error_text = (
-            10054,
-            'An existing connection was forcibly closed by the remote host',
-        )
-
     underlying_error = ssl_err.value.args[0].args[-1]
     err_text = str(underlying_error)
     assert underlying_error.errno == expected_error_code, (
         'The underlying error is {underlying_error!r}'.format(**locals())
     )
     assert expected_error_text in err_text
+
+
+@pytest.mark.parametrize('adapter_type', ('builtin', 'pyopenssl'))
+@pytest.mark.parametrize(
+    'encrypted_key',
+    (True, False),
+    ids=('encrypted-key', 'unencrypted-key'),
+)
+@pytest.mark.parametrize(
+    'password_as_bytes',
+    (True, False),
+    ids=('with-bytes-password', 'with-str-password'),
+)
+# pylint: disable-next=too-many-positional-arguments
+def test_ssl_adapters_with_private_key_password(
+    http_request_timeout,
+    private_key_password,
+    tls_http_server,
+    tls_ca_certificate_pem_path,
+    tls_certificate_chain_pem_path,
+    tls_certificate_passwd_private_key_pem_path,
+    tls_certificate_private_key_pem_path,
+    adapter_type,
+    encrypted_key,
+    password_as_bytes,
+):
+    """Check server decrypts private TLS keys with password as bytes or str."""
+    key_file = (
+        tls_certificate_passwd_private_key_pem_path
+        if encrypted_key
+        else tls_certificate_private_key_pem_path
+    )
+    key_pass = (
+        private_key_password.encode('utf-8')
+        if password_as_bytes
+        else private_key_password
+    )
+
+    tls_adapter_cls = get_ssl_adapter_class(name=adapter_type)
+    tls_adapter = tls_adapter_cls(
+        certificate=tls_certificate_chain_pem_path,
+        private_key=key_file,
+        private_key_password=key_pass,
+    )
+
+    interface, _host, port = _get_conn_data(
+        tls_http_server(
+            (ANY_INTERFACE_IPV4, EPHEMERAL_PORT),
+            tls_adapter,
+        ).bind_addr,
+    )
+
+    resp = requests.get(
+        f'https://{interface!s}:{port!s}/',
+        timeout=http_request_timeout,
+        verify=tls_ca_certificate_pem_path,
+    )
+
+    assert resp.status_code == 200
+    assert resp.text == 'Hello world!'
+
+
+@pytest.mark.parametrize(
+    'adapter_type',
+    ('builtin',),
+)
+def test_builtin_adapter_with_false_key_password(
+    tls_certificate_chain_pem_path,
+    tls_certificate_passwd_private_key_pem_path,
+    adapter_type,
+):
+    """Check that builtin ssl-adapter initialization fails when wrong private key password given."""
+    tls_adapter_cls = get_ssl_adapter_class(name=adapter_type)
+    with pytest.raises(ssl.SSLError, match=r'^\[SSL\] PEM.+'):
+        tls_adapter_cls(
+            certificate=tls_certificate_chain_pem_path,
+            private_key=tls_certificate_passwd_private_key_pem_path,
+            private_key_password='x' * 256,
+        )
+
+
+@pytest.mark.parametrize(
+    ('adapter_type', 'false_password', 'expected_warn'),
+    (
+        (
+            'pyopenssl',
+            '837550fd-bcb9-4320-87e6-09de6456b09',
+            contextlib.nullcontext(),
+        ),
+        ('pyopenssl', 555555, contextlib.nullcontext()),
+        (
+            'pyopenssl',
+            '@' * 2048,
+            pytest.warns(
+                UserWarning,
+                match=r'^User-provided password is 2048 bytes.+',
+            ),
+        ),
+    ),
+    ids=('incorrect-password', 'integer-password', 'too-long-password'),
+)
+def test_openssl_adapter_with_false_key_password(
+    tls_certificate_chain_pem_path,
+    tls_certificate_passwd_private_key_pem_path,
+    adapter_type,
+    false_password,
+    expected_warn,
+):
+    """Check that server init fails when wrong private key password given."""
+    httpserver = HTTPServer(
+        bind_addr=(ANY_INTERFACE_IPV4, EPHEMERAL_PORT),
+        gateway=Gateway,
+    )
+
+    tls_adapter_cls = get_ssl_adapter_class(name=adapter_type)
+    tls_adapter = tls_adapter_cls(
+        certificate=tls_certificate_chain_pem_path,
+        private_key=tls_certificate_passwd_private_key_pem_path,
+        private_key_password=false_password,
+    )
+
+    httpserver.ssl_adapter = tls_adapter
+
+    with expected_warn, pytest.raises(
+        OpenSSL.SSL.Error,
+        # Decode error has happened very rarely with Python 3.9 in MacOS.
+        # Might be caused by a random issue in file handling leading
+        # to interpretation of garbage characters in certificates.
+        match=r'.+\'(bad decrypt|decode error)\'.+',
+    ):
+        httpserver.prepare()
+
+    assert not httpserver.requests._threads
+    assert not httpserver.ready
